package com.jiuqi.util.chinese;

import java.util.ArrayList;
import java.io.Serializable;

public class ChineseInt implements Cloneable, Serializable
{
    private static final char[] INVALID_CHARS;
    private String intValue;
    
    static {
        INVALID_CHARS = new char[] { ' ', '\u3000' };
    }
    
    public ChineseInt() {
        this.intValue = "0";
    }
    
    public ChineseInt(final long value) {
        this.intValue = String.valueOf(value);
    }
    
    public ChineseInt(final String value) {
        if (isIntString(value)) {
            this.intValue = value;
        }
        else {
            try {
                this.intValue = valueOf(value);
            }
            catch (final Exception ex) {
                this.intValue = "0";
            }
        }
    }
    
    public long longValue() {
        return Long.parseLong(this.intValue);
    }
    
    public String chineseValue(final boolean showUnit, final boolean useBigChar) {
        return parseToChinese(this.intValue, showUnit, useBigChar);
    }
    
    public static String valueOf(final String value) throws NotValidChineseNumberException, NoValidCharException {
        boolean negtive = false;
        final StringSplitor chars = new StringSplitor(value, ChineseInt.INVALID_CHARS);
        char thisChar = chars.getNextValidChar();
        if (thisChar == '(' || thisChar == '（') {
            thisChar = chars.getNextValidChar();
            if (thisChar != '负') {
                throw new NotValidChineseNumberException("字符'('后面的字符无效，必须是'负'！");
            }
            thisChar = chars.getNextValidChar();
            if (thisChar != ')' && thisChar != '）') {
                throw new NotValidChineseNumberException("字符'负'后面的字符无效，必须是')'！");
            }
            negtive = true;
        }
        if (!negtive) {
            chars.rollBack();
        }
        boolean hasUnit = false;
        thisChar = chars.getNextValidChar();
        if (ChineseDigit.isValidChineseDigit(thisChar)) {
            thisChar = chars.getNextValidChar();
            if (ChineseDigitUnit.isValidChineseDigitUnit(String.valueOf(thisChar))) {
                hasUnit = true;
            }
            chars.rollBack();
            chars.rollBack();
            long longValue = 0L;
            if (hasUnit) {
                int tempValue = 0;
                while (chars.hasValidChar()) {
                    thisChar = chars.getNextValidChar();
                    if (ChineseDigit.isValidChineseDigit(thisChar)) {
                        try {
                            tempValue = ChineseDigit.valueOf(thisChar);
                        }
                        catch (final NotValidChineseDigitException ex) {
                            throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                        }
                        if (chars.hasValidChar()) {
                            continue;
                        }
                        longValue += tempValue;
                    }
                    else {
                        if (!ChineseDigitUnit.isValidChineseDigitUnit(String.valueOf(thisChar))) {
                            throw new NotValidChineseNumberException("存在非法的字符\"" + thisChar + "\"！");
                        }
                        if (ChineseDigitUnit.isLargeUnit(thisChar)) {
                            longValue += tempValue;
                            tempValue = 0;
                            try {
                                longValue *= Long.parseLong(ChineseDigitUnit.valueOf(String.valueOf(thisChar)));
                                continue;
                            }
                            catch (final NotValidChineseDigitUnitException ex2) {
                                throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                            }
                        }
                        try {
                            longValue += tempValue * Long.parseLong(ChineseDigitUnit.valueOf(String.valueOf(thisChar)));
                        }
                        catch (final NotValidChineseDigitUnitException ex3) {
                            throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                        }
                        tempValue = 0;
                    }
                }
            }
            else {
                final StringBuffer stringValue = new StringBuffer();
                while (chars.hasValidChar()) {
                    thisChar = chars.getNextValidChar();
                    if (ChineseDigit.isValidChineseDigit(thisChar)) {
                        try {
                            stringValue.append(ChineseDigit.valueOf(thisChar));
                        }
                        catch (final NotValidChineseDigitException ex4) {
                            throw new NotValidChineseNumberException("存在非法字符\"" + thisChar + "\"！");
                        }
                    }
                }
                longValue = Long.parseLong(stringValue.toString());
            }
            if (negtive) {
                longValue = -longValue;
            }
            return String.valueOf(longValue);
        }
        throw new NotValidChineseNumberException("开头的字符\"" + thisChar + "\"不是数字！");
    }
    
    public static String parseToChinese(String value, final boolean showUnit, final boolean useBigChar) {
        if (value != null && !value.equals("") && isIntString(value)) {
            boolean negative = false;
            if (value.startsWith("-")) {
                negative = true;
                value = value.substring(1);
            }
            final ArrayList digits = new ArrayList();
            for (int i = 0; i < value.length(); ++i) {
                final int digit = value.charAt(i) - '0';
                if (digit >= 0 && digit <= 9) {
                    digits.add(new Integer(digit));
                }
            }
            final StringBuffer result = new StringBuffer();
            if (!showUnit) {
                for (int j = 0; j < digits.size(); ++j) {
                    final int digit2 = digits.get(j);
                    result.append(ChineseDigit.toChineseChar(digit2, useBigChar));
                }
            }
            else {
                int cursor = digits.size();
                int smallInt = 0;
                int k = 0;
                while (cursor >= 0) {
                    smallInt = getSmallInt(digits, k);
                    if (smallInt != 0) {
                        result.insert(0, String.valueOf(smallIntToChineseString(smallInt, useBigChar)) + ChineseDigitUnit.toChineseUnit(getUnit(k * 4), useBigChar));
                    }
                    cursor -= 4;
                    if (smallInt < 1000 && smallInt > 0 && cursor >= 0) {
                        result.insert(0, ChineseDigit.toChineseChar(0, useBigChar));
                    }
                    ++k;
                }
            }
            if (negative) {
                result.insert(0, "(负)");
            }
            return result.toString();
        }
        throw new IllegalArgumentException("字符串\"" + value + "\"不是合法的阿拉伯数字字符串！");
    }
    
    private static int getSmallInt(final ArrayList digits, final int i) {
        int value = 0;
        for (int j = 0, cursor = digits.size() - i * 4 - 1; j < 4 && cursor >= 0; --cursor, ++j) {
            final int digit = digits.get(cursor);
            value += (int)(digit * Math.pow(10.0, j));
        }
        return value;
    }
    
    private static String smallIntToChineseString(int value, final boolean useBigChar) {
        if (value >= 0 && value < 10000) {
            final int[] digits = new int[4];
            for (int count = 3; value != 0; value /= 10, --count) {
                digits[count] = value % 10;
            }
            final StringBuffer result = new StringBuffer();
            boolean allZeroBefore = true;
            boolean zeroExist = false;
            for (int i = 0; i < digits.length; ++i) {
                if (digits[i] != 0) {
                    result.append(String.valueOf(ChineseDigit.toChineseChar(digits[i], useBigChar)) + ChineseDigitUnit.toChineseUnit(getUnit(3 - i), useBigChar));
                    allZeroBefore = false;
                    zeroExist = false;
                }
                else if (!allZeroBefore) {
                    boolean allZeroAfter = true;
                    for (int j = i + 1; j < 4; ++j) {
                        if (digits[j] != 0) {
                            allZeroAfter = false;
                            break;
                        }
                    }
                    if (!allZeroAfter && !zeroExist) {
                        result.append(ChineseDigit.toChineseChar(0, useBigChar));
                        zeroExist = true;
                    }
                }
            }
            return result.toString();
        }
        throw new IllegalArgumentException("只能解析0-9999的整数");
    }
    
    static boolean isIntString(String value) {
        if (value.startsWith("-")) {
            value = value.substring(1);
        }
        return ChineseDigitUnit.isDigitString(value);
    }
    
    private static String getUnit(final int i) {
        final StringBuffer sb = new StringBuffer("1");
        for (int j = 0; j < i; ++j) {
            sb.append("0");
        }
        return sb.toString();
    }
    
    public Object clone() {
        return new ChineseInt(this.intValue);
    }
    
    @Override
    public boolean equals(final Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof ChineseInt)) {
            return false;
        }
        final ChineseInt value = (ChineseInt)o;
        return this.intValue.equals(value.intValue);
    }
    
    @Override
    public int hashCode() {
        int result = super.hashCode();
        if (this.intValue != null) {
            result ^= this.intValue.hashCode();
        }
        return result;
    }
    
    @Override
    public String toString() {
        return this.chineseValue(true, false);
    }
}
